import { Role } from './ad.types';
import { PointProps } from './value-objects/point.value-object';
import { ScheduleItemProps } from './value-objects/schedule-item.value-object';
import { CarpoolStepProps } from './value-objects/carpool-step.value-object';
import { JourneyProps } from './value-objects/journey.value-object';
import { StepProps } from './value-objects/step.value-object';

// All properties that a Candidate has
export interface CandidateProps {
  role: Role;
  driverWaypoints: PointProps[];
  passengerWaypoints: PointProps[];
  driverDistance: number;
  driverDuration: number;
  carpoolSteps?: CarpoolStepProps[];
  distance?: number;
  duration?: number;
  steps?: StepProps[];
  driverSchedule: ScheduleItemProps[];
  passengerSchedule: ScheduleItemProps[];
  journeys?: JourneyProps[];
  spacetimeDetourRatio: SpacetimeDetourRatio;
}

// Properties that are needed for a Candidate creation
export interface CreateCandidateProps {
  id: string;
  role: Role;
  driverDistance: number;
  driverDuration: number;
  driverWaypoints: PointProps[];
  passengerWaypoints: PointProps[];
  driverSchedule: ScheduleItemProps[];
  passengerSchedule: ScheduleItemProps[];
  spacetimeDetourRatio: SpacetimeDetourRatio;
}

export enum Target {
  START = 'START',
  INTERMEDIATE = 'INTERMEDIATE',
  FINISH = 'FINISH',
  NEUTRAL = 'NEUTRAL',
}

export abstract class Validator {
  abstract validate(): boolean;
}

export type SpacetimeMetric = {
  distance: number;
  duration: number;
};

export type SpacetimeDetourRatio = {
  maxDistanceDetourRatio: number;
  maxDurationDetourRatio: number;
};
