import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';
import { Actor } from './actor.value-object';
import { Role } from '../ad.types';
import { Point } from './point.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface CarpoolStepProps {
  point: Point;
  actors: Actor[];
}

export class CarpoolStep extends ValueObject<CarpoolStepProps> {
  get point(): Point {
    return this.props.point;
  }

  get actors(): Actor[] {
    return this.props.actors;
  }

  protected validate(props: CarpoolStepProps): void {
    if (props.actors.length <= 0)
      throw new ArgumentOutOfRangeException('at least one actor is required');
    if (
      props.actors.filter((actor: Actor) => actor.role == Role.DRIVER).length >
      1
    )
      throw new ArgumentOutOfRangeException(
        'a carpoolStep can contain only one driver',
      );
  }
}
