import { ArgumentOutOfRangeException } from '@mobicoop/ddd-library';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { Actor } from '@modules/ad/core/domain/value-objects/actor.value-object';
import { Point } from '@modules/ad/core/domain/value-objects/point.value-object';
import { CarpoolStep } from '@modules/ad/core/domain/value-objects/carpool-step.value-object';

describe('CarpoolStep value object', () => {
  it('should create a carpoolStep value object', () => {
    const carpoolStepVO = new CarpoolStep({
      point: new Point({
        lat: 48.689445,
        lon: 6.17651,
      }),
      actors: [
        new Actor({
          role: Role.DRIVER,
          target: Target.NEUTRAL,
        }),
        new Actor({
          role: Role.PASSENGER,
          target: Target.START,
        }),
      ],
    });
    expect(carpoolStepVO.point.lon).toBe(6.17651);
    expect(carpoolStepVO.point.lat).toBe(48.689445);
    expect(carpoolStepVO.actors).toHaveLength(2);
  });
  it('should throw an exception if actors is empty', () => {
    try {
      new CarpoolStep({
        point: new Point({
          lat: 48.689445,
          lon: 6.17651,
        }),
        actors: [],
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
  it('should throw an exception if actors contains more than one driver', () => {
    try {
      new CarpoolStep({
        point: new Point({
          lat: 48.689445,
          lon: 6.17651,
        }),
        actors: [
          new Actor({
            role: Role.DRIVER,
            target: Target.NEUTRAL,
          }),
          new Actor({
            role: Role.DRIVER,
            target: Target.START,
          }),
        ],
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
});
